@@ -827,4 +827,34 @@ sv_res_table <- function(fit_res, format_out = FALSE) {
   }
   
   return(df_out)
+}
+
+#' Segment T1 weighted MRI data using FSL FAST and write to file. Runs deface
+#' and bet as preprocessing steps by default.
+#' 
+#' This function requires a working installation of FSL and uses the fslr 
+#' package. You may need to specify the fsl install directory, eg: 
+#' 'options(fsl.path = "/path/to/fsl")'
+#' 
+#' @param mri_path path to the volumetric T1 data.
+#' @param deface deface the input T1 data before analysis. Defaults to TRUE.
+#' @param bet_fit fractional intensity threshold for bet brain extraction.
+#' Values should be between 0 and 1. Defaults to 0.5 with smaller values giving
+#' larger brain estimates.
+#' @export 
+segment_t1_fsl <- function(mri_path, deface = TRUE, bet_fit = 0.5) {
+  dir_path <- dirname(mri_path)
+  if (deface) {
+    deface_path <- file.path(dir_path, "t1_deface")
+    fslr::fsl_deface(mri_path, outfile = deface_path, retimg = FALSE,
+                     verbose = FALSE)
+    mri_path <- deface_path
+  }
+  brain_path <- file.path(dir_path, "t1_brain")
+  bet_opts <- paste("-B -f", bet_fit)
+  fslr::fslbet(mri_path, outfile = brain_path, retimg = FALSE, opts = bet_opts,
+               betcmd = "bet", verbose = FALSE)
+  seg_prefix <- file.path(dir_path, "t1")
+  fslr::fslfast(brain_path, outfile = seg_prefix, retimg = FALSE,
+                verbose = FALSE)
 }
\ No newline at end of file
